# frozen_string_literal: true

module PlanetExpress
  class ShippingOptionRepository < Repository
    attr_reader :data

    def initialize(data: nil)
      @data = data || default_data
      calculate_provider_size_lookup_table
      calculate_lowest_price_lookup_table
    end

    def find_by_provider_and_package_size(provider, package_size)
      provider_size_lookup_table[[provider, package_size]]&.first
    end

    def lowest_price_for_package_size(package_size)
      lowest_price_lookup_table[package_size]
    end

    private

    attr_reader :provider_size_lookup_table, :lowest_price_lookup_table

    def default_data
      [
        ShippingOption.new('LP', 'S', 150),
        ShippingOption.new('LP', 'M', 490),
        ShippingOption.new('LP', 'L', 690),
        ShippingOption.new('MR', 'S', 200),
        ShippingOption.new('MR', 'M', 300),
        ShippingOption.new('MR', 'L', 400)
      ]
    end

    def calculate_provider_size_lookup_table
      @provider_size_lookup_table = data.group_by do |option|
        [option.provider, option.package_size]
      end
    end

    def calculate_lowest_price_lookup_table
      available_package_sizes = data.map { |option| option.package_size }.uniq
      lowest_price_per_option_tuples = available_package_sizes.map do |package_size|
        option = data.select { |option| option.package_size == package_size }
            .min_by { |option| option.price_in_cents }

        [option.package_size, option.price_in_cents]
      end

      @lowest_price_lookup_table = Hash[lowest_price_per_option_tuples]
    end
  end
end
