# frozen_string_literal: true

RSpec.describe PlanetExpress::ShipmentRepository do
  let(:items) { [] }
  subject { described_class.new(data: items) }

  describe 'rule_applied_in_month?' do
    context 'empty repository' do
      let(:items) { [] }

      it 'returns false' do
        result = subject.rule_applied_in_month?(Date.today.begin_of_month,
                                                PlanetExpress::Rule)

        expect(result).to eq(false)
      end
    end

    context 'with invalid shipments' do
      let(:items) { [PlanetExpress::InvalidShipment.new('')] }
      it 'returns false' do
        result = subject.rule_applied_in_month?(Date.today.begin_of_month,
                                                PlanetExpress::Rule)

        expect(result).to eq(false)
      end
    end

    context 'with valid shipment matching' do
      let(:rule) do
        Object.new
      end

      let(:shipment) do
        shipment = PlanetExpress::Shipment.new(Date.today, build(:shipping_option))
        shipment.add_discount(PlanetExpress::Discount.new(50, rule))
        shipment
      end

      let(:items) { [shipment] }

      it 'returns true' do
        result = subject.rule_applied_in_month?(Date.today.begin_of_month,
                                                rule.class)

        expect(result).to eq(true)
      end
    end

    context 'with valid shipment not matching' do
      let(:rule) do
        Object.new
      end

      let(:other_rule) do
        -> {}
      end

      let(:shipment) do
        shipment = PlanetExpress::Shipment.new(Date.today, build(:shipping_option))
        shipment.add_discount(PlanetExpress::Discount.new(50, rule))
        shipment
      end

      let(:items) { [shipment] }

      it 'returns false' do
        result = subject.rule_applied_in_month?(Date.today.begin_of_month,
                                                other_rule.class)

        expect(result).to eq(false)
      end
    end
  end

  describe '#l_shipment_count_for_provider_and_month' do
    context 'empty repository' do
      let(:items) { [] }

      it 'returns 0' do
        result = subject.l_shipment_count_for_provider_and_month(
          'LP',
          Date.today.begin_of_month
        )

        expect(result).to eq(0)
      end
    end

    context 'with invalid shipments' do
      let(:items) { [PlanetExpress::InvalidShipment.new('')] }
      it 'returns 0' do
        result = subject.l_shipment_count_for_provider_and_month(
          'LP',
          Date.today.begin_of_month
        )

        expect(result).to eq(0)
      end
    end

    context 'with valid shipment matching' do
      let(:shipment) do
        PlanetExpress::Shipment.new(Date.today,
                                    build(:shipping_option,
                                          package_size: 'L'))
      end

      let(:provider) { shipment.provider }
      let(:items) { [shipment] }

      it 'returns 1' do
        result = subject.l_shipment_count_for_provider_and_month(provider,
                                                                 Date.today.begin_of_month)

        expect(result).to eq(1)
      end
    end

    context 'with valid shipment not matching' do
      let(:shipment) do
        PlanetExpress::Shipment.new(Date.today,
                                    build(:shipping_option,
                                          package_size: 'L'))
      end

      let(:provider) { shipment.provider }
      let(:items) { [shipment] }

      context 'different provider' do
        it 'returns 0' do
          result = subject.l_shipment_count_for_provider_and_month('MP_XXX_TEST',
                                                                   Date.today.begin_of_month)

          expect(result).to eq(0)
        end
      end

      context 'different month' do
        it 'returns 0' do
          result = subject.l_shipment_count_for_provider_and_month(provider,
                                                                   Date.today.next_month.begin_of_month)

          expect(result).to eq(0)
        end
      end
    end
  end
end
